package
{
	import flash.geom.Point;
	
	import fairygui.GButton;
	import fairygui.GComponent;
	import fairygui.GObject;
	import fairygui.GRoot;
	import fairygui.event.GTouchEvent;
	import fairygui.tween.EaseType;
	import fairygui.tween.GTween;
	import fairygui.tween.GTweener;
	
	import starling.events.EventDispatcher;

	public class JoystickModule extends EventDispatcher
	{
		private var _InitX: Number;
		private var _InitY: Number;
		private var _startStageX: Number;
		private var _startStageY: Number;
		private var _lastStageX: Number;
		private var _lastStageY: Number;
		private var _button: GButton;
		private var _touchArea: GObject;
		private var _thumb: GObject;
		private var _center: GObject;
		private var _touchId: Number;
		private var _tweener: GTweener;
		private var _curPos:Point;
		private var _radius: Number;
		
		public static const JoystickMoving: String = "JoystickMoving";
		public static const JoystickUp: String = "JoystickUp";
		
		public function JoystickModule(mainView: GComponent):void {
			super();
			
			this._button = mainView.getChild("joystick").asButton;
			this._button.changeStateOnClick = false;
			this._thumb = this._button.getChild("thumb");
			this._touchArea = mainView.getChild("joystick_touch");
			this._center = mainView.getChild("joystick_center");
			
			this._InitX = this._center.x + this._center.width / 2;
			this._InitY = this._center.y + this._center.height / 2;
			this._touchId = -1;
			this._radius = 150;
			
			this._curPos = new Point();
			
			this._touchArea.addEventListener(GTouchEvent.BEGIN,this.onTouchDown);
		}
		
		public function trigger(evt: GTouchEvent): void {
			this.onTouchDown(evt);
		}
		
		private function onTouchDown(evt: GTouchEvent):void {
			if(this._touchId == -1)//First touch
			{
				this._touchId = evt.touchPointID;
				
				if(this._tweener != null) {
					this._tweener.kill();
					this._tweener = null;
				}
				
				GRoot.inst.globalToLocal(evt.stageX,evt.stageY,this._curPos);
				var bx: Number = this._curPos.x;
				var by: Number = this._curPos.y;
				this._button.selected = true;
				
				if(bx < 0)
					bx = 0;
				else if(bx > this._touchArea.width)
					bx = this._touchArea.width;
				
				if(by > fairygui.GRoot.inst.height)
					by = fairygui.GRoot.inst.height;
				else if(by < this._touchArea.y)
					by = this._touchArea.y;
				
				this._lastStageX = bx;
				this._lastStageY = by;
				this._startStageX = bx;
				this._startStageY = by;
				
				this._center.visible = true;
				this._center.x = bx - this._center.width / 2;
				this._center.y = by - this._center.height / 2;
				this._button.x = bx - this._button.width / 2;
				this._button.y = by - this._button.height / 2;
				
				var deltaX: Number = bx - this._InitX;
				var deltaY: Number = by - this._InitY;
				var degrees: Number = Math.atan2(deltaY,deltaX) * 180 / Math.PI;
				this._thumb.rotation = degrees + 90;
				
				this._touchArea.addEventListener(GTouchEvent.DRAG,this.onTouchMove);
				this._touchArea.addEventListener(GTouchEvent.END,this.onTouchUp);
			}
		}
		
		private function onTouchUp(evt: GTouchEvent): void {
			if(this._touchId != -1 && evt.touchPointID == this._touchId) {
				this._touchId = -1;
				this._thumb.rotation = this._thumb.rotation + 180;
				this._center.visible = false; 
				this._tweener = GTween.to2(this._button.x, this._button.y, this._InitX - this._button.width / 2, this._InitY - this._button.height / 2, 0.3)
					.setTarget(this._button, this._button.setXY)
					.setEase(EaseType.QuadOut)
					.onComplete(__tweenComplete);
				
				this._touchArea.removeEventListener(GTouchEvent.DRAG,this.onTouchMove);
				this._touchArea.removeEventListener(GTouchEvent.END,this.onTouchUp);
				
				this.dispatchEventWith(JoystickModule.JoystickUp);
			}
		}
		
		private function __tweenComplete():void
		{
			this._tweener = null;
			this._button.selected = false;
			this._thumb.rotation = 0;
			this._center.visible = true;
			this._center.x = this._InitX - this._center.width / 2;
			this._center.y = this._InitY - this._center.height / 2;
		}
		
		private function onTouchMove(evt: GTouchEvent): void {
			if(this._touchId != -1 && evt.touchPointID == this._touchId) {
				var bx: Number = evt.stageX / fairygui.GRoot.contentScaleFactor;
				var by: Number = evt.stageY / fairygui.GRoot.contentScaleFactor;
				var moveX: Number = bx - this._lastStageX;
				var moveY: Number = by - this._lastStageY;
				this._lastStageX = bx;
				this._lastStageY = by;
				var buttonX: Number = this._button.x + moveX;
				var buttonY: Number = this._button.y + moveY;
				
				var offsetX: Number = buttonX + this._button.width / 2 - this._startStageX;
				var offsetY: Number = buttonY + this._button.height / 2 - this._startStageY;
				
				var rad: Number = Math.atan2(offsetY,offsetX);
				var degree: Number = rad * 180 / Math.PI;
				this._thumb.rotation = degree + 90;
				
				var maxX: Number = this._radius * Math.cos(rad);
				var maxY: Number = this._radius * Math.sin(rad);
				if(Math.abs(offsetX) > Math.abs(maxX))
					offsetX = maxX;
				if(Math.abs(offsetY) > Math.abs(maxY))
					offsetY = maxY;
				
				buttonX = this._startStageX + offsetX;
				buttonY = this._startStageY + offsetY;
				if(buttonX < 0)
					buttonX = 0;
				if(buttonY > fairygui.GRoot.inst.height)
					buttonY = fairygui.GRoot.inst.height;
				
				this._button.x = buttonX - this._button.width / 2;
				this._button.y = buttonY - this._button.height / 2;
				
				this.dispatchEventWith(JoystickModule.JoystickMoving,false,degree);
			}
		}
	}
}